/*
CoreListener.java
Copyright (C) 2010  Belledonne Communications, Grenoble, France

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

package org.linphone.core;

import androidx.annotation.Nullable;
import androidx.annotation.NonNull;

/**
* That class holds all the callbacks which are called by LinphoneCore. <br/>
* <br/>
* Once created, add your LinphoneCoreCbs using linphone_core_add_listener. Keep a<br/>
* reference on it as long as you need it. You can use<br/>
* linphone_core_remove_listener to remove it but that isn't mandatory.<br/>
* The same applies to all listeners in our API. <br/>
*/
public interface CoreListener {
    /**
    * Chat message callback prototype. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param chatRoom LinphoneChatRoom involved in this conversation. Can be created<br/>
    * by the framework in case the From-URI is not present in any chat room.   <br/>
    * @param message LinphoneChatMessage incoming message   <br/>
    */
    public void onMessageReceived(@NonNull Core core, @NonNull ChatRoom chatRoom, @NonNull ChatMessage message);

    /**
    * Callback prototype for configuring status changes notification. <br/>
    * <br/>
    * @param core the LinphoneCore   <br/>
    * @param status the current LinphoneConfiguringState <br/>
    * @param message informational message.   <br/>
    */
    public void onConfiguringStatus(@NonNull Core core, ConfiguringState status, @Nullable String message);

    /**
    * Chat room marked as read callback. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param chatRoom LinphoneChatRoom that has been marked as read.   <br/>
    */
    public void onChatRoomRead(@NonNull Core core, @NonNull ChatRoom chatRoom);

    /**
    * Callback notifying that a new LinphoneCall (either incoming or outgoing) has<br/>
    * been created. <br/>
    * <br/>
    * @param core LinphoneCore object that has created the call   <br/>
    * @param call The newly created LinphoneCall object   <br/>
    */
    public void onCallCreated(@NonNull Core core, @NonNull Call call);

    /**
    * Callback prototype telling a call has started (incoming or outgoing) while<br/>
    * there was no other call. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    */
    public void onFirstCallStarted(@NonNull Core core);

    /**
    * Call encryption changed callback. <br/>
    * <br/>
    * @param core the LinphoneCore   <br/>
    * @param call the LinphoneCall on which encryption is changed.   <br/>
    * @param mediaEncryptionEnabled whether encryption is activated. <br/>
    * @param authenticationToken an authentication_token, currently set for ZRTP kind<br/>
    * of encryption only.   <br/>
    */
    public void onCallEncryptionChanged(@NonNull Core core, @NonNull Call call, boolean mediaEncryptionEnabled, @Nullable String authenticationToken);

    /**
    * Callback prototype for reporting the result of a version update check. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param result The result of the version update check   <br/>
    * @param url The url where to download the new version if the result is<br/>
    * #LinphoneVersionUpdateCheckNewVersionAvailable   <br/>
    */
    public void onVersionUpdateCheckResultReceived(@NonNull Core core, @NonNull VersionUpdateCheckResult result, String version, @Nullable String url);

    /**
    * Called after the #send method of the LinphoneChatMessage was called. <br/>
    * <br/>
    * The message will be in state InProgress. In case of resend this callback won't<br/>
    * be called. <br/>
    * @param core LinphoneCore object   <br/>
    * @param chatRoom LinphoneChatRoom involved in this conversation. Can be be<br/>
    * created by the framework in case the From-URI is not present in any chat room. <br/>
    *  <br/>
    * @param message LinphoneChatMessage outgoing message   <br/>
    */
    public void onMessageSent(@NonNull Core core, @NonNull ChatRoom chatRoom, @NonNull ChatMessage message);

    /**
    * Callback prototype telling that a LinphoneChatRoom state has changed. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param chatRoom The LinphoneChatRoom object for which the state has changed   <br/>
    * @param state the current LinphoneChatRoomState <br/>
    */
    public void onChatRoomStateChanged(@NonNull Core core, @NonNull ChatRoom chatRoom, ChatRoom.State state);

    /**
    * Callback prototype for receiving info messages. <br/>
    * <br/>
    * @param core the LinphoneCore   <br/>
    * @param call the call whose info message belongs to.   <br/>
    * @param message the info message.   <br/>
    */
    public void onInfoReceived(@NonNull Core core, @NonNull Call call, @NonNull InfoMessage message);

    /**
    * Callback prototype telling that a LinphoneChatRoom ephemeral message has<br/>
    * expired. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param chatRoom The LinphoneChatRoom object for which a message has expired.   <br/>
    */
    public void onChatRoomEphemeralMessageDeleted(@NonNull Core core, @NonNull ChatRoom chatRoom);

    /**
    * Callback prototype telling that a LinphoneChatRoom subject has changed. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param chatRoom The LinphoneChatRoom object for which the subject has changed   <br/>
    */
    public void onChatRoomSubjectChanged(@NonNull Core core, @NonNull ChatRoom chatRoom);

    /**
    * Call state notification callback. <br/>
    * <br/>
    * @param core the LinphoneCore   <br/>
    * @param call the LinphoneCall object whose state is changed.   <br/>
    * @param state the new LinphoneCallState of the call <br/>
    * @param message a non null informational message about the state.   <br/>
    */
    public void onCallStateChanged(@NonNull Core core, @NonNull Call call, Call.State state, @NonNull String message);

    /**
    * Reports presence model change for a specific URI or phone number of a friend. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param linphoneFriend LinphoneFriend object   <br/>
    * @param uriOrTel The URI or phone number for which the presence model has<br/>
    * changed   <br/>
    * @param presenceModel The new LinphonePresenceModel   <br/>
    */
    public void onNotifyPresenceReceivedForUriOrTel(@NonNull Core core, @NonNull Friend linphoneFriend, @NonNull String uriOrTel, @NonNull PresenceModel presenceModel);

    /**
    * Reports that a new subscription request has been received and wait for a<br/>
    * decision. <br/>
    * <br/>
    * note: A subscription request is notified by this function only if the<br/>
    * LinphoneSubscribePolicy for the given LinphoneFriend has been set to<br/>
    * LinphoneSPWait. See linphone_friend_set_inc_subscribe_policy. <br/>
    * @param core LinphoneCore object   <br/>
    * @param linphoneFriend The LinphoneFriend aimed by the subscription.   <br/>
    * @param url URI of the subscriber   <br/>
    */
    public void onNewSubscriptionRequested(@NonNull Core core, @NonNull Friend linphoneFriend, @NonNull String url);

    /**
    * Function prototype used by #linphone_core_cbs_set_ec_calibrator_audio_uninit(). <br/>
    * <br/>
    * @param core The LinphoneCore.   <br/>
    */
    public void onEcCalibrationAudioUninit(@NonNull Core core);

    /**
    * Callback prototype telling the last call has ended<br/>
    * (#LinphoneCore.get_calls_nb() returns 0) <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    */
    public void onLastCallEnded(@NonNull Core core);

    /**
    * Callback prototype when using the buddy plugin. <br/>
    * <br/>
    * @param core the LinphoneCore   <br/>
    * @param linphoneFriend the LinphoneFriend that has been updated   <br/>
    */
    public void onBuddyInfoUpdated(@NonNull Core core, @NonNull Friend linphoneFriend);

    /**
    * Report status change for a friend previously added to the LinphoneCore with<br/>
    * linphone_core_add_friend(). <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param linphoneFriend Updated LinphoneFriend   <br/>
    */
    public void onNotifyPresenceReceived(@NonNull Core core, @NonNull Friend linphoneFriend);

    /**
    * Callback notifying that a LinphoneAccount has its registration state changed. <br/>
    * <br/>
    * @param core The LinphoneCore object.   <br/>
    * @param account The LinphoneAccount object which has its registration changed.   <br/>
    * @param state The new state for this account. <br/>
    * @param message a non null informational message about the state   <br/>
    */
    public void onAccountRegistrationStateChanged(@NonNull Core core, @NonNull Account account, RegistrationState state, @NonNull String message);

    /**
    * Callback prototype telling the audio devices list has been updated. <br/>
    * <br/>
    * Either a new device is available or a previously available device isn't<br/>
    * anymore. You can call linphone_core_get_audio_devices to get the new list. <br/>
    * @param core LinphoneCore object   <br/>
    */
    public void onAudioDevicesListUpdated(@NonNull Core core);

    /**
    * Callback prototype for reporting log collection upload progress indication. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param offset the number of bytes sent since the start of the upload <br/>
    * @param total the total number of bytes to upload <br/>
    */
    public void onLogCollectionUploadProgressIndication(@NonNull Core core, int offset, int total);

    /**
    * Global state notification callback. <br/>
    * <br/>
    * @param core the LinphoneCore.   <br/>
    * @param state the LinphoneGlobalState <br/>
    * @param message informational message.   <br/>
    */
    public void onGlobalStateChanged(@NonNull Core core, GlobalState state, @NonNull String message);

    /**
    * Callback prototype for when a refer is received. <br/>
    * <br/>
    * @param core the LinphoneCore   <br/>
    * @param referTo the address of the refer   <br/>
    */
    public void onReferReceived(@NonNull Core core, @NonNull String referTo);

    /**
    * Callback prototype for reporting network change either automatically detected<br/>
    * or notified by linphone_core_set_network_reachable. <br/>
    * <br/>
    * @param core the LinphoneCore   <br/>
    * @param reachable true if network is reachable. <br/>
    */
    public void onNetworkReachable(@NonNull Core core, boolean reachable);

    /**
    * Callback to notify a new call-log entry has been added. <br/>
    * <br/>
    * This is done typically when a call terminates. <br/>
    * @param core the LinphoneCore   <br/>
    * @param callLog the new LinphoneCallLog entry added.   <br/>
    */
    public void onCallLogUpdated(@NonNull Core core, @NonNull CallLog callLog);

    /**
    * Callback prototype telling that a LinphoneConference state has changed. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param conference The LinphoneConference object for which the state has changed<br/>
    *   <br/>
    * @param state the current LinphoneChatRoomState <br/>
    */
    public void onConferenceStateChanged(@NonNull Core core, @NonNull Conference conference, Conference.State state);

    /**
    * Callback prototype telling that an Instant Message Encryption Engine user<br/>
    * registered on the server with or without success. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param status the return status of the registration action. <br/>
    * @param userId the userId published on the encryption engine server   <br/>
    * @param info information about failure   <br/>
    */
    public void onImeeUserRegistration(@NonNull Core core, boolean status, @NonNull String userId, @NonNull String info);

    /**
    * Callback for requesting authentication information to application or user. <br/>
    * <br/>
    * @param core the LinphoneCore   <br/>
    * @param authInfo a LinphoneAuthInfo pre-filled with username, realm and domain<br/>
    * values as much as possible   <br/>
    * @param method the type of authentication requested as LinphoneAuthMethod enum  <br/>
    * Application shall reply to this callback using linphone_core_add_auth_info. <br/>
    */
    public void onAuthenticationRequested(@NonNull Core core, @NonNull AuthInfo authInfo, @NonNull AuthMethod method);

    /**
    * Callback for notifying progresses of transfers. <br/>
    * <br/>
    * @param core the LinphoneCore   <br/>
    * @param transfered the LinphoneCall that was transfered   <br/>
    * @param callState the LinphoneCallState of the call to transfer target at the<br/>
    * far end. <br/>
    */
    public void onTransferStateChanged(@NonNull Core core, @NonNull Call transfered, Call.State callState);

    /**
    * Callback for being notified of DTMFs received. <br/>
    * <br/>
    * @param core the LinphoneCore   <br/>
    * @param call the LinphoneCall that received the dtmf   <br/>
    * @param dtmf the ascii code of the dtmf <br/>
    */
    public void onDtmfReceived(@NonNull Core core, @NonNull Call call, int dtmf);

    /**
    * Function prototype used by #linphone_core_cbs_set_ec_calibrator_result(). <br/>
    * <br/>
    * @param core The LinphoneCore.   <br/>
    * @param status The LinphoneEcCalibratorStatus of the calibrator. <br/>
    * @param delayMs The measured delay if available. <br/>
    */
    public void onEcCalibrationResult(@NonNull Core core, EcCalibratorStatus status, int delayMs);

    /**
    * Callback prototype for notifying the application about notification received<br/>
    * from the network. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param linphoneEvent the LinphoneEvent received   <br/>
    * @param notifiedEvent The event as string   <br/>
    * @param body the LinphoneContent of the event   <br/>
    */
    public void onNotifyReceived(@NonNull Core core, @NonNull Event linphoneEvent, @NonNull String notifiedEvent, @NonNull Content body);

    /**
    * Function prototype used by #linphone_core_cbs_set_ec_calibrator_audio_init(). <br/>
    * <br/>
    * @param core The LinphoneCore.   <br/>
    */
    public void onEcCalibrationAudioInit(@NonNull Core core);

    /**
    * Callback prototype telling that the audio device for at least one call has<br/>
    * changed. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param audioDevice the newly used LinphoneAudioDevice object   <br/>
    */
    public void onAudioDeviceChanged(@NonNull Core core, @NonNull AudioDevice audioDevice);

    /**
    * Is composing notification callback prototype. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param chatRoom LinphoneChatRoom involved in the conversation.   <br/>
    */
    public void onIsComposingReceived(@NonNull Core core, @NonNull ChatRoom chatRoom);

    /**
    * Callback prototype for reporting when a friend list has been removed from the<br/>
    * core friends list. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param friendList LinphoneFriendList object   <br/>
    */
    public void onFriendListRemoved(@NonNull Core core, @NonNull FriendList friendList);

    /**
    * Callback prototype for notifying the application about changes of publish<br/>
    * states. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param linphoneEvent the LinphoneEvent   <br/>
    * @param state the new LinphonePublishState <br/>
    */
    public void onPublishStateChanged(@NonNull Core core, @NonNull Event linphoneEvent, PublishState state);

    /**
    * Registration state notification callback prototype. <br/>
    * <br/>
    * @param core the LinphoneCore   <br/>
    * @param proxyConfig the LinphoneProxyConfig which state has changed   <br/>
    * @param state the current LinphoneRegistrationState <br/>
    * @param message a non null informational message about the state   <br/>
    * @deprecated 06/04/2020 Use LinphoneCoreCbsAccountRegistrationStateChangedCb<br/>
    * instead<br/>
    */
    public void onRegistrationStateChanged(@NonNull Core core, @NonNull ProxyConfig proxyConfig, RegistrationState state, @NonNull String message);

    /**
    * Callback for receiving quality statistics for calls. <br/>
    * <br/>
    * @param core the LinphoneCore   <br/>
    * @param call the call   <br/>
    * @param callStats the call statistics.   <br/>
    */
    public void onCallStatsUpdated(@NonNull Core core, @NonNull Call call, @NonNull CallStats callStats);

    /**
    * Callback prototype for reporting log collection upload state change. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param state The state of the log collection upload <br/>
    * @param info Additional information: error message in case of error state, URL<br/>
    * of uploaded file in case of success.   <br/>
    */
    public void onLogCollectionUploadStateChanged(@NonNull Core core, Core.LogCollectionUploadState state, @NonNull String info);

    /**
    * Callback prototype for notifying the application about changes of subscription<br/>
    * states, including arrival of new subscriptions. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param linphoneEvent the LinphoneEvent   <br/>
    * @param state the new LinphoneSubscriptionState <br/>
    */
    public void onSubscriptionStateChanged(@NonNull Core core, @NonNull Event linphoneEvent, SubscriptionState state);

    /**
    * Chat message not decrypted callback prototype. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param chatRoom LinphoneChatRoom involved in this conversation. Can be be<br/>
    * created by the framework in case the from-URI is not present in any chat room. <br/>
    *  <br/>
    * @param message LinphoneChatMessage incoming message   <br/>
    */
    public void onMessageReceivedUnableDecrypt(@NonNull Core core, @NonNull ChatRoom chatRoom, @NonNull ChatMessage message);

    /**
    * Callback prototype telling the result of decoded qrcode. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param result The result of the decoded qrcode   <br/>
    */
    public void onQrcodeFound(@NonNull Core core, @Nullable String result);

    /**
    * Callback prototype for reporting when a friend list has been added to the core<br/>
    * friends list. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param friendList LinphoneFriendList object   <br/>
    */
    public void onFriendListCreated(@NonNull Core core, @NonNull FriendList friendList);

    /**
    * Callback prototype for notifying the application about subscription received<br/>
    * from the network. <br/>
    * <br/>
    * @param core LinphoneCore object   <br/>
    * @param linphoneEvent the LinphoneEvent received   <br/>
    * @param subscribeEvent The event as string   <br/>
    * @param body the LinphoneContent of the event   <br/>
    */
    public void onSubscribeReceived(@NonNull Core core, @NonNull Event linphoneEvent, @NonNull String subscribeEvent, @NonNull Content body);

    /**
    * Callback to notify the callid of a call has been updated. <br/>
    * <br/>
    * This is done typically when a call retry. <br/>
    * @param core the LinphoneCore   <br/>
    * @param previousCallId the previous callid.   <br/>
    * @param currentCallId the new callid.   <br/>
    */
    public void onCallIdUpdated(@NonNull Core core, @NonNull String previousCallId, @NonNull String currentCallId);

}